@@ -55,7 +55,8 @@ try:
                 fd1 = os.dup(s.fileno())
                 fd2 = os.dup(s.fileno())
                 s.close()
-                sp = subprocess.Popen([path.exe(), 'mux', '--', 'server']
+                sp = subprocess.Popen([path.exe(), 'mux', '--',
+                                       path.exe(), 'server']
                                       + extra, stdin=fd1, stdout=fd2)
             finally:
                 os.close(fd1)
