@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 import sys, os, re
 from bup import options
+from bup import _helpers   # fixes up sys.argv on import
 
 optspec = """
 bup newliner
@@ -14,6 +15,7 @@ if extra:
 r = re.compile(r'([\r\n])')
 lastlen = 0
 all = ''
+width = options._tty_width() or 78
 while 1:
     l = r.split(all, 1)
     if len(l) <= 1:
@@ -31,7 +33,8 @@ while 1:
     else:
         assert(len(l) == 3)
         (line, splitchar, all) = l
-        #splitchar = '\n'
+        if splitchar == '\r':
+            line = line[:width]
         sys.stdout.write('%-*s%s' % (lastlen, line, splitchar))
         if splitchar == '\r':
             lastlen = len(line)
@@ -39,5 +42,7 @@ while 1:
             lastlen = 0
         sys.stdout.flush()
 
-if lastlen or all:
-    sys.stdout.write('%-*s\n' % (lastlen, all))
+if lastlen:
+    sys.stdout.write('%-*s\r' % (lastlen, ''))
+if all:
+    sys.stdout.write('%s\n' % all)
